package org.jboss.qa.drools.example;

import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.server.UnicastRemoteObject;

import org.jboss.qa.drools.example.server.Drools;

public class DroolsExample implements DroolsCommand {
    public static final String name = "my-drools";

    public static void main(String[] args) throws Exception {
        DroolsCommand command = new DroolsExample();

        DroolsCommand stub = (DroolsCommand)UnicastRemoteObject.exportObject(command, 0);
        LocateRegistry.getRegistry().rebind(name, stub);
        System.out.println("Started...");
    }

    @Override
    public void shutdown() throws RemoteException {
        System.out.println("Quiting...");
        new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    LocateRegistry.getRegistry().unbind(name);
                    System.exit(0);
                } catch (Exception ex) {
                    System.err.println(ex);
                }
            }

        }).start();
    }

    //////////////////////////////////////////// Drools ////////////////////////////////////////////////
    private final Drools helper = new Drools();
    public void createKnowledgeBase(String id) throws RemoteException {
        try {
            helper.createBase(id);
        } catch (Throwable t) {
            throw new RemoteException("Error creating knowledge base", t);
        }
    }

    public void createKnowledgeSession(String idBase, String id) throws RemoteException {
        try {
            helper.getBase(idBase).createSession(id);
        } catch (Throwable t) {
            throw new RemoteException("Error creating knowledge session", t);
        }
    }

    public void insertFact(String idSession, String id, Object fact) throws RemoteException {
        try {
            helper.findSessionByName(idSession).insertFact(id, fact);
        } catch (Throwable t) {
            throw new RemoteException("Error inserting fact", t);
        }
    }

    public void updateFact(String idObject, Object fact) throws RemoteException {
        try {
            helper.findSessionByFact(idObject).updateFact(idObject, fact);
        } catch (Throwable t) {
            throw new RemoteException("Error updating fact", t);
        }
    }

    public void retractFact(String idObject) throws RemoteException {
        try {
            helper.findSessionByFact(idObject).retractFact(idObject);
        } catch (Throwable t) {
            throw new RemoteException("Error updating fact", t);
        }
    }

    public void insertEvent(String idSession, String entrypoint, Object event) throws RemoteException {
        helper.findSessionByName(idSession).insertEvent(entrypoint, event);
    }

    public void fireRules(String idSession) throws RemoteException {
        helper.findSessionByName(idSession).fireRules();
    }

    public void disposeSession(String idSession) throws RemoteException {
        helper.findBaseBySession(idSession).disposeSession(idSession);
    }

}
