package org.jboss.qa.drools.example;

import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;


public class DroolsRemoteController {
    private static final int MODE_HELLO = 99;
    private static final int MODE_SHUTDOWN = 0;
    private static final int MODE_PHASE1 = 1;
    private static final int MODE_PHASE2 = 2;


    public static void main(String[] args) throws Exception {
        DroolsCommand stub = (DroolsCommand)LocateRegistry.getRegistry().lookup(DroolsExample.name);

        if (args.length < 1) {
            System.out.println("No mode selected!");
            return;
        }

        int mode = Integer.parseInt(args[0]);

        switch(mode) {
            case MODE_HELLO:
                stub.createKnowledgeBase("kb");
                stub.createKnowledgeSession("kb", "session");
                stub.insertFact("session", null, "Hello world!");
                break;

            case MODE_PHASE1:
                phase1(stub);
                break;

            case MODE_PHASE2:
                phase2(stub);
                break;

            case MODE_SHUTDOWN:
                stub.shutdown();
                break;

            default:
                System.err.println("Unknown mode: " + mode);
        }

        System.out.println("Finished...");
    }

    static final String idBase = "my-own-knowledge-base";
    static final String idSession = "my-little-session";
    static final String idFact = "fact";

    private static void phase1(DroolsCommand stub) throws RemoteException {
        System.out.println("Starting drools engine...");
        stub.createKnowledgeBase(idBase);
        stub.createKnowledgeSession(idBase, idSession);
        stub.insertFact(idSession, null, "");
        stub.insertFact(idSession, idFact, "removable fact!");
    }

    private static void phase2(DroolsCommand stub) throws RemoteException {
        stub.retractFact(idFact);
        stub.fireRules(idSession);

        stub.disposeSession(idSession);

        stub.createKnowledgeSession(idBase, idSession);
        stub.fireRules(idSession);
    }
}
