package org.jboss.qa.drools.example.server;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.drools.KnowledgeBase;

public class Base {
    private final Map<String, Session> sessions;
    private final KnowledgeBase kbase;

    public Base(KnowledgeBase kbase) {
        this.kbase = kbase;
        this.sessions = new HashMap<String, Session>();
    }

    public Session createSession(String idSession) {
        if (hasSession(idSession)) {
            throw new IllegalArgumentException("Duplicate Session: " + idSession);
        }
        Session session = new Session(kbase.newStatefulKnowledgeSession());

        sessions.put(idSession, session);

        return session;
    }

    public Collection<String> getSessionNames() {
        return Collections.unmodifiableSet(sessions.keySet());
    }

    Collection<Session> getSessions() {
        return Collections.unmodifiableCollection(sessions.values());
    }

    public void disposeSession(String idSession) {
        if (!hasSession(idSession)) {
            throw new IllegalArgumentException("Nonexistent Session: " + idSession);
        }

        sessions.remove(idSession).dispose();
    }

    public boolean hasSession(String idSession) {
        return sessions.containsKey(idSession);
    }

    public Session getSession(String idSession) {
        return sessions.get(idSession);
    }
}
