package org.jboss.qa.drools.example.server;

import java.util.HashMap;
import java.util.Map;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseConfiguration;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.conf.EventProcessingOption;
import org.drools.conf.MBeansOption;
import org.drools.io.ResourceFactory;

public class Drools {
    private final Map<String, Base> bases = new HashMap<String, Base>();

    public boolean hasBase(String idBase) {
        return bases.containsKey(idBase);
    }

    public Base getBase(String idBase) {
        return bases.get(idBase);
    }

    public Base createBase(String idBase) {
        if (hasBase(idBase)) {
            throw new IllegalArgumentException("Duplicate KnowledgeBase: " + idBase);
        }
        Base base = new Base(createNewKnowledgeBase(idBase));

        bases.put(idBase, base);

        return base;
    }

    public Base findBaseBySession(String idSession) {
        for (Base b : bases.values()) {
            if (b.hasSession(idSession)) {
                return b;
            }
        }

        return null;
    }

    public Session findSessionByName(String idSession) {
        for (Base b : bases.values()) {
            if (b.hasSession(idSession)) {
                return b.getSession(idSession);
            }
        }

        return null;
    }

    public Session findSessionByFact(String idFact) {
        for (Base b : bases.values()) {
            for (Session s : b.getSessions()) {
                if (s.hasFact(idFact)) {
                    return s;
                }
            }
        }

        return null;
    }

    private KnowledgeBase createNewKnowledgeBase(String id) {
        KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();


        // TODO rules
        kbuilder.add(ResourceFactory.newClassPathResource("rules.drl"), ResourceType.DRL);


        if (kbuilder.hasErrors()) {
            throw new RuntimeException(kbuilder.getErrors().toString());
        }

        KnowledgeBaseConfiguration kbconf = KnowledgeBaseFactory.newKnowledgeBaseConfiguration();
        kbconf.setOption(MBeansOption.ENABLED);
        kbconf.setOption(EventProcessingOption.STREAM);

        KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase(id, kbconf);
        kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());

        return kbase;
    }
}
