package org.jboss.qa.drools.example.server;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.rule.FactHandle;

public class Session {
    private final StatefulKnowledgeSession ksession;
    private final Map<String, FactHandle> facts;

    public Session(StatefulKnowledgeSession ksession) {
        this.ksession = ksession;
        this.facts = new HashMap<String, FactHandle>();
    }

    public FactHandle insertFact(String idFact, Object fact) {
        if (hasFact(idFact)) {
            throw new IllegalArgumentException("Duplicate fact: " + idFact);
        }
        FactHandle handle = ksession.insert(fact);

        facts.put(idFact, handle);

        return handle;
    }

    public void updateFact(String idFact, Object fact) {
        if (!hasFact(idFact)) {
            throw new IllegalArgumentException("Nonexistent fact: " + idFact);
        }

        ksession.update(getFact(idFact), fact);
    }

    public void retractFact(String idFact) {
        if (!hasFact(idFact)) {
            throw new IllegalArgumentException("Nonexistent fact: " + idFact);
        }

        ksession.retract(facts.remove(idFact));
    }

    public void insertEvent(String entrypoint, Object event) {
        ksession.getWorkingMemoryEntryPoint(entrypoint).insert(event);
    }

    public void fireRules() {
        ksession.fireAllRules();
    }

    void dispose() {
        facts.clear();
        ksession.dispose();
    }

    public Collection<String> getFactNames() {
        return Collections.unmodifiableSet(facts.keySet());
    }

    Collection<FactHandle> getFacts() {
        return Collections.unmodifiableCollection(facts.values());
    }

    public boolean hasFact(String idFact) {
        return facts.containsKey(idFact);
    }

    public FactHandle getFact(String idFact) {
        return facts.get(idFact);
    }
}
